@@ -18,12 +18,22 @@ struct GraphingTexture
 	
 	void setPixel(int x, int y, unsigned char red, unsigned char green, unsigned char blue, unsigned char alpha)
 	{
-		m_imageData[x*4+y*4*m_width+0] = red;
-		m_imageData[x*4+y*4*m_width+1] = green;
-		m_imageData[x*4+y*4*m_width+2] = blue;
-		m_imageData[x*4+y*4*m_width+3] = alpha;
+		if (y>=0 && y<m_height && x>=0 && x<m_width)
+		{
+			m_imageData[x*4+y*4*m_width+0] = red;
+			m_imageData[x*4+y*4*m_width+1] = green;
+			m_imageData[x*4+y*4*m_width+2] = blue;
+			m_imageData[x*4+y*4*m_width+3] = alpha;
+		}
 	}
 	
+	void getPixel(int x, int y, unsigned char& red, unsigned char& green, unsigned char& blue, unsigned char& alpha)
+	{
+		red = m_imageData[x*4+y*4*m_width+0];
+		green = m_imageData[x*4+y*4*m_width+1];
+		blue = m_imageData[x*4+y*4*m_width+2];
+		alpha = m_imageData[x*4+y*4*m_width+3];
+	}
 	void uploadImageData();
 	
 	int getTextureId()
