@@ -3,15 +3,21 @@
 
 #include "../CommonInterfaces/CommonMultiBodyBase.h"
 
-class SharedMemoryCommon : public CommonMultiBodyBase
+class SharedMemoryCommon : public CommonExampleInterface
 {
+protected:
+	
+	struct GUIHelperInterface* m_guiHelper;
+	
 public:
 	SharedMemoryCommon(GUIHelperInterface* helper)
-	:CommonMultiBodyBase(helper)
+	:m_guiHelper(helper)
 	{
 	}
 
+	virtual void setSharedMemoryKey(int key)=0;
 	virtual bool wantsTermination()=0;
+    virtual bool isConnected()=0;
 };
 #endif//
 
